/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2012 digitalSTROM.org, Zurich, Switzerland
 *  Author: Roman Köhler
 */

function getNameForSensorType(sensorType) {
	switch (sensorType) {
	 	case 4:
	 		return _('active power');
	 	case 5:
	 		return _('output current');
	 	case 6:
	 		return _('electric meter');
	 	case 9:
	 		return _('temperature indoor');
	 	case 10:
	 		return _('temperature outdoor');
	 	case 11:
	 		return _('brightness indoors');
	 	case 12:
	 		return _('brightness outdoors');
	 	case 13:
	 		return _('huminity indoors');
	 	case 14:
	 		return _('huminity outdoors');
	 	case 15:
	 		return _('Air pressure');
	 	case 18:
	 		return _('wind speed');
	 	case 19:
	 		return _('wind direction');
	 	case 20:
	 		return _('Precipitation');
	 	case 21:
	 		return _('carbon dioxide');
	 	case 25:
	 		return _('sound level');
	 	case 50:
	 		return _('nominal temperature');
	 	case 51:
	 		return _('heating valve value');
	 	case 64:
	 		return _('Output current');
	 	case 65:
	 		return _('Power consumption');
	 }
	 return 'type ' + sensorType;
}

function getParameterByName(name)
{
  name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
  var regexS = "[\\?&]" + name + "=([^&#]*)";
  var regex = new RegExp(regexS);
  var results = regex.exec(window.location.search);
  if(results == null)
    return "";
  else
    return decodeURIComponent(results[1].replace(/\+/g, " "));
}

Ext.define('DSS.addon.HLEEdit', {

    extend : 'DSS.addon.Framework',

    config : {
        appName : 'User defined action',
        appId : 'hleedit',
        appVersion : '1.4.4.0',
        appIcon : 'default_icon.png'
    },

    getHelp : function() {
        return Ext.create('Ext.Component', {
            whiteSpace : 'normal',
            padding : '0 20 0 0', //top right bottom left
            loader : {
                url: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad : true,
                disableCaching : false
            }
        });
    },
    makeUDSPanel: function () {
    	var me=this;
    
    	var oConfigName={
                defaultSortCol:'name',
                storeConfig:{
                    storeId : 'udsStore',
                    fields : [ {name:'id'}, 
                               {name:'name', sortType:'asUCText'}] 
                },
                multiselect:true,
                columns:[              
                    {
                        header : _('Name'),
                        dataIndex : 'name',
                        xtype: 'textcolumn',
                        flex:1
                    }
                ],
                buttonText: {
                    newButton: _('New state'),
                    editButton: _('Edit state'),
                    deleteButton: _('Delete state')
                },
                editorConfig:{
                    title : _('Edit user defined state'),
                    staticControls : [ {
                        type : 'text',
                        label : _('Name'),
                        id : 'name'
                    }, {
                        type : 'hidden',
                        label : 'id',
                        id : 'id'
                    } ]
                },
                listeners:{
                    'loadDataStore':function (grid){
                        me.loadParameterState(grid);
                    },
                    'saveEntry':function (grid,values){                        

                        me.waitAndReloadState(grid, {
                            actions : 'generateSensorState',
                            value : Ext.JSON.encode(values)
                        });
                    },
                    'deleteEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReloadState(grid, {
                            actions : 'delete',
                            id : Ext.JSON.encode(idVal)
                        });
                    }
                }
            };
                    
            return Ext.create('DSS.gridPanel', oConfigName);
    }, 
    makeSensorStatePanel: function () {
    	var me=this;
        var meme=null;
    	var oConfigName={
                defaultSortCol:'name',
                 storeConfig:{
                    storeId : 'udsSensorStore',
                    fields : [ {name:'id'}, 
                               {name:'name', sortType:'asUCText'},
                               {name:'target'},
                               {name:'active'},
                               {name:'inactive'},
                               {name:'activevalue'},
                               {name:'inactivevalue'},
                               {name:'type'},
                               {name:'target_device'},
                               {name:'target_zone'}] 
                },
                multiselect:true,
                columns:[              
                         {
                             header : _('Name'),
                             dataIndex : 'name',
                             xtype: 'textcolumn',
                             flex:1
                         },{
                             header : _('Sensorvalue'),
                             dataIndex : 'target',
                             xtype: 'textcolumn',
                             flex:1,
                             customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
                            	 if (val.type=='dsid') {
                            		 var oDevice=dss.staticDataModel.getDevice(val.dsid);
                            		 var sString = '<table><tr><td style="padding-right:5px;"><img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '"></td><td>';
                            		 sString = oDevice.name + _(" in ");
                                     sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                                     return sString + '</td></tr></table>';
                            	 } else {
                            		 var sString='<table><tr>'
                            		 if (val.group != 0)
                                         sString += '<td style="padding-right:5px;"><img src="'+dss.staticDataModel.getImagePathForGroup(val.zone,val.group)+'"></td><td>' + dss.staticDataModel.getNameForGroup(val.zone,val.group) + _(" in "); 
                                     if (val.zone != 0)
                                         sString += dss.staticDataModel.getNameForZone(val.zone);
                                     return sString + '</td></tr></table>'; 
                            	 }
                             }
                         },{
                             header : _('Sensortype'),
                             dataIndex : 'type',
                             xtype: 'textcolumn',
                             customColumnRenderer : getNameForSensorType,
                             flex:1
                         },{
                             header : _('Active condition'),
                             dataIndex : 'active',
                             xtype: 'textcolumn',
                             flex:1,
                             customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
                            	 return unescape(val).replace(';','');
                             }
                         },{
                             header : _('Inactive condition'),
                             dataIndex : 'inactive',
                             xtype: 'textcolumn',
                             flex:1,
                             customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
                            	 return unescape(val).replace(';','');
                             }
                         }
                ],
                buttonText: {
                    newButton: _('New sensor state'),
                    editButton: _('Edit sensor state'),
                    deleteButton: _('Delete senor state')
                },
                editorConfig:{
                    title : _('Edit sensor state'),
                    staticControls : [ {
                        type : 'text',
                        label : _('Name'),
                        id : 'name'
                    }, {
                        type : 'hidden',
                        label : 'id',
                        id : 'id'
                    } ,{
                    	type: 'custom',
                    	id:'sensorSource',
                    	control:Ext.create('Ext.form.RadioGroup', {
                            columns:2,
                            fieldLabel: _('sensor source'),
                            items: [
                                    {style:{color:'#005138','margin-bottom':0},width:200,boxLabel:_('sensor from device'),name:'sensorSource',inputValue:0},
                                    {style:{color:'#005138','margin-bottom':0},width:200,boxLabel:_('sensor on zone'),name:'sensorSource',inputValue:1}
                            ],
                            listeners:{
                            	change:function(mthis,value) {
                            		if (value==null) {
                            			me.megrid.editorWindow.editControls.target_zone.setDisabled(true);
                            			me.megrid.editorWindow.editControls.target_device.setDisabled(true);
                            		} else {
                            			me.megrid.editorWindow.editControls.target_zone.setDisabled(value.sensorSource==0);
                            			me.megrid.editorWindow.editControls.target_device.setDisabled(value.sensorSource!=0);
                            		}
                            	}
                            }
                        })                    		
                    },  {
                        type : 'combobox',
                        label : _('Sensor on Zone'),
                        id : 'target_zone',
                        storevalueid:'_id',
                        storedisplayid:'name',
                        storeid:'zoneSensorBase'
                    },  {
                        type : 'combobox',
                        label : _('Sensor on Device'),
                        id : 'target_device',
                        storevalueid:'_id',
                        storedisplayid:'name',
                        storeid:'deviceSensorBase'
                    }, {
                    	type: 'custom',
                    	control:Ext.create('Ext.form.field.Number', {
                            name : 'activeID',
                            allowBlank : false,
                            blankText : _('This field is required'),
                            invalidText : _('The value in this field is invalid'),
                            fieldLabel : _('activate on'),
                            value : 0
                        }),
                        id: 'activevalue'
                    }, {
                    	type: 'custom',
                    	control:Ext.create('Ext.form.field.Number', {
                            name : 'inactiveID',
                            allowBlank : false,
                            blankText : _('This field is required'),
                            invalidText : _('The value in this field is invalid'),
                            fieldLabel : _('inactivate on'),
                            value : 0
                        }),
                        id: 'inactivevalue'
                    }]
                },
                listeners:{
                    'loadDataStore':function (grid){
                        me.loadParameterSensorState(grid);
                    },
                    'saveEntry':function (grid,values){                        
                    	var oValues={
                    		id:values.id,
                    		name:values.name,
                    		activate: "> ; "+ values.activevalue,
                    		inactive: "< ; "+ values.inactivevalue
                    	};
                    	if (values.sensorSource.sensorSource==1) {
                    		var target=values.target_zone.split('_');
                    		oValues.zoneID=target[0];
                    		oValues.groupID=target[1];
                    		oValues.sensorType=target[2];
                    	} else {
                    		var target=values.target_device.split('_');
                    		oValues.dsid=target[0];
                    		oValues.sensorType=target[1];
                    	}
                        me.waitAndReloadSensorState(grid, {
                            actions : 'generateSensorState',
                            value : Ext.JSON.encode(oValues)
                        });
                    },
                    'deleteEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReloadSensorState(grid, {
                            actions : 'delete',
                            id : Ext.JSON.encode(idVal)
                        });
                    }
                }
            };
    	this.megrid= Ext.create('DSS.gridPanel', oConfigName);
		me.megrid.editorWindow.editControls.target_zone.setDisabled(true);
		me.megrid.editorWindow.editControls.target_device.setDisabled(true);
    	
        return this.megrid;
    },
    megrid:null,
    getContent : function() {
        var me=this;

        var conditions = [{type:'singleItem',title:_('state_daynight'),entry:[
                              {type:'state',name:'daylight',value:true,title:Ext.String.format2(_('daynight_bright'),{
                                        duskTime:dss.staticDataModel.getDusk(),
                                        dawnTime:dss.staticDataModel.getDawn()
                                    }),gridText:Ext.String.format2(_('grid_daynight_twilight_true'),{
                                        duskTime:dss.staticDataModel.getDusk(),
                                        dawnTime:dss.staticDataModel.getDawn()
                                    })},
                              {type:'state',name:'daylight',value:false,title:Ext.String.format2(_('daynight_dark'),{
                                    duskTime:dss.staticDataModel.getDusk(),
                                    dawnTime:dss.staticDataModel.getDawn()
                                }),gridText:Ext.String.format2(_('grid_daynight_twilight_false'),{
                                    duskTime:dss.staticDataModel.getDusk(),
                                    dawnTime:dss.staticDataModel.getDawn()
                                })},
                              {type:'state',name:'daynight',value:true,title:Ext.String.format2(_('daynight_day'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                }),gridText:Ext.String.format2(_('grid_daynight_daynight_true'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                })},
                              {type:'state',name:'daynight',value:false,title:Ext.String.format2(_('daynight_night'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                }),gridText:Ext.String.format2(_('grid_daynight_daynight_false'),{
                                    sunriseTime:dss.staticDataModel.getSunrise(),
                                    sunsetTime:dss.staticDataModel.getSunset()
                                })}]},
                          {type:'timeframe',title:_('Timeframe')},
                          {type:'weekdays',title:_('Weekdays')},
                          {type:'dateframe',title:_('Dateframe')},
                          {type:'deviceState',title:_('automatic devices')},
                          {type:'singleItem',title:_('state_presence'),entry: [
                              {type:'state',name:'presence',value:'present',title:_('presence_present'),gridText:_('grid_presence_present')},
                              {type:'state',name:'presence',value:'absent',title:_('presence_absent'),gridText:_('grid_presence_absent')}]},
                          {type:'singleItem',title:_('state_holiday'),entry: [
                              {type:'state',name:'holiday',value:'on',title:_('holiday_on'),gridText:_('grid_holiday_on')},
                              {type:'state',name:'holiday',value:'off',title:_('holiday_off'),gridText:_('grid_holiday_off')}]},
                          {type:'multiItem',title:_('weather trigger'),entry: [
                              {type:'state',name:'rain',value:'1',title:_('rain active'),gridText:_('grid_rain_active'),group:1},
                              {type:'state',name:'rain',value:'2',title:_('rain inactive'),gridText:_('grid_rain_inactive'),group:1},
                              {type:'state',name:'wind',value:'1',title:_('wind active'),gridText:_('grid_wind_active'),group:2},
                              {type:'state',name:'wind',value:'2',title:_('wind inactive'),gridText:_('grid_wind_inactive'),group:2},
                              {type:'state',name:'hail',value:'1',title:_('hail active'),gridText:_('grid_hail_active'),group:3},
                              {type:'state',name:'hail',value:'2',title:_('hail inactive'),gridText:_('grid_hail_inactive'),group:3},
                              {type:'state',name:'frost',value:'1',title:_('frost active'),gridText:_('grid_frost_active'),group:4},
                              {type:'state',name:'frost',value:'2',title:_('frost inactive'),gridText:_('grid_frost_inactive'),group:4},
                              {type:'state',name:'sun',value:'1',title:_('sun active'),gridText:_('grid_sun_active'),group:5},
                              {type:'state',name:'sun',value:'2',title:_('sun inactive'),gridText:_('grid_sun_inactive'),group:5}]},
                              {type:'state',name:'service',value:'1',title:_('service active'),gridText:_('grid_service_active'),group:6},
                              {type:'state',name:'service',value:'2',title:_('service inactive'),gridText:_('grid_service_inactive'),group:6},
                         {type:'multiItem',title:_('Security'),entry: [
                              {type:'state',name:'fire',value:'1',title:_('fire active'),gridText:_('grid_fire_active'),group:1},
                              {type:'state',name:'fire',value:'2',title:_('fire inactive'),gridText:_('grid_fire_inactive'),group:1},
                              {type:'state',name:'panic',value:'1',title:_('panic_active'),gridText:_('grid_panic_active'),group:2},
                              {type:'state',name:'panic',value:'2',title:_('panic_inactive'),gridText:_('grid_panic_inactive'),group:2},
                              {type:'state',name:'alarm',value:'1',title:_('alarm_active'),gridText:_('grid_alarm_active'),group:3},
                              {type:'state',name:'alarm',value:'2',title:_('alarm_inactive'),gridText:_('grid_alarm_inactive'),group:3},
                              {type:'state',name:'alarm2',value:'1',title:_('alarm2_active'),gridText:_('grid_alarm2_active'),group:4},
                              {type:'state',name:'alarm2',value:'2',title:_('alarm2_inactive'),gridText:_('grid_alarm2_inactive'),group:4},
                              {type:'state',name:'alarm3',value:'1',title:_('alarm3_active'),gridText:_('grid_alarm3_active'),group:5},
                              {type:'state',name:'alarm3',value:'2',title:_('alarm3_inactive'),gridText:_('grid_alarm3_inactive'),group:5},
                              {type:'state',name:'alarm4',value:'1',title:_('alarm4_active'),gridText:_('grid_alarm4_active'),group:6},
                              {type:'state',name:'alarm4',value:'2',title:_('alarm4_inactive'),gridText:_('grid_alarm4_inactive'),group:6}]},
                          {type:'zonestate',title:_('ZoneState')},
                          {type:'clusterstate',title:_('Cluster State')}];

        var climateSettings = dss.staticDataModel.getClimateSettings();
        if (climateSettings != null && climateSettings.capabilities != null && climateSettings.capabilities.coolingSupported == true) {
            conditions.push({type:'multiItem',title:_('Climate system'),entry: [
                                {type:'state',name:'heating_water_system',value:'0',title:_('off'),gridText:_('grid_climate_off'),group:1},
                                {type:'state',name:'heating_water_system',value:'1',title:_('heating on'),gridText:_('grid_climate_heating'),group:1},
                                {type:'state',name:'heating_water_system',value:'2',title:_('cooling on'),gridText:_('grid_climate_cooling'),group:1}]
            });
        } else {
            conditions.push({type:'multiItem',title:_('Climate system'),entry: [
                                {type:'state',name:'heating_water_system',value:'0',title:_('off'),gridText:_('grid_climate_off'),group:1},
                                {type:'state',name:'heating_water_system',value:'1',title:_('heating on'),gridText:_('grid_climate_heating'),group:1}]
            });
        }

        var oConfig={
            defaultSortCol:'name',
            disabledProperty:'disabled',            
            storeConfig:{
                storeId : 'hleStore',
                fields : [ {name:'id'}, 
                           {name:'disabled', sortType:'asBoolean'}, 
                           {name:'name', sortType:'asUCText'}, 
                           {name:'actions', sortType:'asAction'}, 
                           {name:'lastSaved', sortType:'asTimestamp'}, 
                           {name:'lastExecuted', sortType:'asTimestamp'},
                           {name: 'conditions',  sortType: 'asCondition' },
                           {name:'source', sortType:'asUCText'}]
            },
            multiselect:true,
            columns:[
                {
                    header : _('Active'),
                    dataIndex : 'disabled',
                    resizable : false,
                    xtype: 'disabledcolumn', 
                    width : 50
                },{
                    header : _('Name'),
                    dataIndex : 'name',
                    xtype: 'textcolumn',
                    width : 200
                },{
                    header : _('Activity'),
                    dataIndex : 'actions', 
                    maintainFlex : true,
                    flex : 5,
                    xtype: 'actioncolumn'
                },{
                    header: _('Conditions'),
                    dataIndex: 'conditions', 
                    xtype: 'conditioncolumn',
                    maintainFlex: true,
                    flex: 3,
                    customRendererOptions:{hideEnabled:true,hideWeekdays:false,hideGenericSystemStates:false,hideZoneStates:false,hideTimeframe:false,filterSystemState:[],
                    	stateTitles:[{name:'daylight',value:true,title:_('grid_daylight_true')},{name:'daylight',value:false,title:_('grid_daylight_false')},
                    	             {name:'daynight',value:true,title:_('grid_daynight_true')},{name:'daynight',value:false,title:_('grid_daynight_false')},
                    	             {name:'panic',value:'1',title:_('grid_panic_active')},{name:'panic',value:'2',title:_('grid_panic_inactive')},
                    	             {name:'presence',value:'present',title:_('grid_presence_present')},{name:'presence',value:'absent',title:_('grid_presence_absent')},
                    	             {name:'fire',value:'1',title:_('grid_fire_active')},{name:'fire',value:'2',title:_('grid_fire_inactive')},
                    	             {name:'hail',value:'1',title:_('grid_hail_active')},{name:'hail',value:'2',title:_('grid_hail_inactive')},
                    	             {name:'wind',value:'1',title:_('grid_wind_active')},{name:'wind',value:'2',title:_('grid_wind_inactive')},
                    	             {name:'rain',value:'1',title:_('grid_rain_active')},{name:'rain',value:'2',title:_('grid_rain_inactive')},
                    	             {name:'frost',value:'1',title:_('grid_frost_active')},{name:'frost',value:'2',title:_('grid_frost_inactive')},
                    	             {name:'sun',value:'1',title:_('grid_sun_active')},{name:'sun',value:'2',title:_('grid_sun_inactive')},
                    	             {name:'service',value:'1',title:_('grid_service_active')},{name:'service',value:'2',title:_('grid_service_inactive')},
                    	             {name:'alarm',value:'1',title:_('grid_alarm_active')},{name:'alarm',value:'2',title:_('grid_alarm_inactive')},
                    	             {name:'alarm2',value:'1',title:_('grid_alarm2_active')},{name:'alarm2',value:'2',title:_('grid_alarm2_inactive')},
                    	             {name:'alarm3',value:'1',title:_('grid_alarm3_active')},{name:'alarm3',value:'2',title:_('grid_alarm3_inactive')},
                    	             {name:'alarm4',value:'1',title:_('grid_alarm4_active')},{name:'alarm4',value:'2',title:_('grid_alarm4_inactive')},
                    	             {name:'holiday', value:'on',title:_('grid_holiday_on')},{name:'holiday', value:'off',title:_('grid_holiday_off')},
                                     {name:'heating_water_system', value:'0',title:_('grid_climate_off')},{name:'heating_water_system', value:'1',title:_('grid_climate_heating')},{name:'heating_water_system', value:'2',title:_('grid_climate_cooling')}]}
                },{
                    header : _('last edit'),
                    dataIndex : 'lastSaved',
                    xtype: 'timestampcolumn',
                    width : 200
                }/*,
                {
                    header : _('last executed'),
                    dataIndex : 'lastExecuted',
                    xtype:'timestampcolumn',
                    width : 200
                }*/
            ],
            buttonText: {
                newButton: _('New action'),
                editButton: _('Edit action'),
                enableButton:_('Activate action'),
                disableButton:_('Deactivate action'),
                testButton: _('Test action'),
                deleteButton: _('Delete action')
            },
            editorConfig:{
                title : _('Edit user defined action'),
                staticControls : [ {
                    type : 'text',
                    label : _('Name'),
                    id : 'name'
                }, {
                    type : 'hidden',
                    label : 'id',
                    id : 'id'
                },{
                    type : 'hidden',
                    label : 'source',
                    id : 'source'
                }],
                tabControls : [ {
                    tabLabel : _('Activity'),
                    tabDescription : _('Choose the activities, which should be executed.'),
                    content : [ {
                        type : 'ActionConfigurator',
                        id : 'actions',
                    	heatingOperationMode:true,
                        flex : true,
                        noHighlevel:true,
                        withDelay : true
                    } ]
                }, {
                    tabLabel : _('Conditions'),
                    content : [{
                        type : 'ConditionConfigurator',
                        id : 'conditions',
                        tabDescription : _('Choose the conditions.'),
                            conditions:conditions,
                            flex : true
                    }]
                } ]
            },
            listeners:{
                'loadDataStore':function (grid){
                    me.loadParameter(grid);
                },
                'saveEntry':function (grid,values){                        
                    me.waitAndReload(grid,{
                        actions : 'save',
                        value : Ext.JSON.encode(values)
                    });
                },
                'enableEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}
                    me.waitAndReload(grid,{
                        actions : 'enable',
                        id : Ext.JSON.encode(idVal)
                    });
                 },
                'disableEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}
                    me.waitAndReload(grid,{
                        actions : 'disable',
                        id : Ext.JSON.encode(idVal)
                    });
                },
                'deleteEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}
                    me.waitAndReload(grid,{
                        actions : 'delete',
                        id : Ext.JSON.encode(idVal)
                    });
                },
                'testEntry':function (grid,entry){
                    dss.raiseEvent('highlevelevent', {
                        id : entry.get('id')
                    });
                    window.setTimeout(function() {
                        me.loadParameter(grid);
                    }, 2 * 1000);
                },
                'selectionChanged': function(grid, selections) {
                	var fSource=(selections.length>0);
                	for (var iIndex=0;iIndex<selections.length;iIndex++) {
                		if (selections[iIndex].get('source')!=null)
                			if (selections[iIndex].get('source')!='')
                				if (selections[iIndex].get('source')!='system-addon-user-defined-actions')
                					 if (getParameterByName('mode')!='expert')
                						 fSource=false;
                	}
                    grid.btnEdit.setDisabled(!fSource);                            
                    grid.btnDelete.setDisabled(!fSource);                      
                }

            }
        };
        
        if (getParameterByName('mode')=='expert') {
        	this.loadSensorBase();
            oConfig.editorConfig.tabControls[0].content[0].enableUsergroup=true;
            oConfig.editorConfig.tabControls[0].content[0].withDelay=true;
            oConfig.editorConfig.tabControls[0].content[0].allowNoAction=true;
            oConfig.editorConfig.tabControls[0].content[0].noHighlevel=false;
            oConfig.editorConfig.tabControls[0].content[0].heatingOperationMode=true;
            oConfig.editorConfig.tabControls[0].content[0].userStates=true;
            oConfig.editorConfig.tabControls[0].content[0].UMRExtendetFunctions=true;
            oConfig.editorConfig.tabControls[0].content[0].heatingExtendedOperationMode=true;
            oConfig.editorConfig.tabControls[1].content[0].ignorePresence=true;
            oConfig.editorConfig.tabControls[1].content[0].conditions.push(
            		{type:'userDefinedState'});
            oConfig.editorConfig.tabControls[1].content[0].fFeatureEnabled={
                    enabler:true,
                    weekdays:true,
                    timeframe:true,
                    systemstate:true,
                    zonestate:true,
                    userDefinedStates:true};

            oConfig.columns.push({
                header : _('source'),
                dataIndex : 'source',
                xtype:'textcolumn',
                width : 200
            })
            
            var statePanel=this.makeUDSPanel();
            var UDAPanel=Ext.create('DSS.gridPanel', oConfig);
            var sensorPanel=this.makeSensorStatePanel();
            return Ext.create('Ext.tab.Panel', {
                layout : 'fit',
                cls:'generalTab',
                items : [ Ext.create('Ext.panel.Panel', { title : _('User defined action'), layout : 'fit', items : [ UDAPanel ] }) ,
                          Ext.create('Ext.panel.Panel', { title : _('user defined states'), layout : 'fit',  items : [ statePanel ]}) ,
                          Ext.create('Ext.panel.Panel', { title : _('sensor states'), layout : 'fit',  items : [ sensorPanel ]}) ]
            });
             
            
        } else 
        	return Ext.create('DSS.gridPanel', oConfig);
    },
    waitAndReload : function(grid,parameter) {
    	grid.setStatus(_('Saving ...'));
    	
    	var me = this;
    	dss.pushAndQuery('system-addon-user-defined-actions-config',parameter,
    			         'system-addon-user-defined-actions-saved',function(eventData) {
    		 me.loadParameter(grid);
    		 return true;
    	});
  
    },
    waitAndReloadState : function(grid,parameter) {
    	grid.setStatus(_('Saving ...'));
    	
    	var me = this;
    	dss.pushAndQuery('user-defined-states.config',parameter,
    			         'user-defined-states.event',function(eventData) {
    		 me.loadParameterState(grid);
    		 return true;
    	});
       
    },
    waitAndReloadSensorState :function(grid,parameter) {
    	grid.setStatus(_('Saving ...'));
    	
    	var me = this;
    	dss.pushAndQuery('user-defined-states.config',parameter,
    			         'user-defined-states.event',function(eventData) {
    		 me.loadParameterSensorState(grid);
    		 return true;
    	});
    },
    loadParameter : function(grid) {
    	grid.setStatus(_('Loading ...'));
        dss.staticDataModel.initHLECache();
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/usr/events/*(*)/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        
        var oResultObject=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                               {name:'source',type:'simple'},
                                                               {name:'id',type:'simple'},
                                                               {name:'disabled',type:'condition/enabled'},
                                                               {name:'lastSaved',type:'simple'},
                                                               {name:'lastExecuted',type:'simple'},
                                                               {name: 'conditions', type: 'condition'},
                                                               {name:'actions',type:'action'}]);
        if (oResultObject!=null) {
            grid.loadParameter(oResultObject);
        }
        grid.setStatus(_('Ready'));
    },
    loadParameterState: function (grid) {
    	var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/user-defined-states/entries/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        
        var oResultObject= dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                               {name:'id',type:'simple'}]);
        if (oResultObject!=null) {
            grid.loadParameter(oResultObject);
        }
        grid.setStatus(_('Ready'));
    },
    loadParameterSensorState: function (grid) {
    	var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/user-defined-states/sensors/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        
        var oResultObject= dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                                {name:'dsid',type:'simple'},
                                                                {name:'group',type:'simple'},
                                                                {name:'zone',type:'simple'},
                                                                {name:'activeValue',type:'simple'},
                                                                {name:'inactiveValue',type:'simple'},
                                                                {name:'sensorType',type:'simple'},
                                                                {name:'id',type:'simple'}]);
        if (oResultObject!=null) {
        	var oArray=[];
        	for (var iIndex=0;iIndex<oResultObject.length;iIndex++) {
        		var oObject={
        				id:oResultObject[iIndex].id,
        				name:oResultObject[iIndex].name,
        				active:oResultObject[iIndex].activeValue,
        				inactive:oResultObject[iIndex].inactiveValue,
        				type:parseInt('' + oResultObject[iIndex].sensorType,10)
        			};
        		if (oResultObject[iIndex].dsid!=null) {
        			oObject.target={type:'device',dsid:oResultObject[iIndex].dsid};
        			oObject.target_device=oResultObject[iIndex].dsid + '_' + oResultObject[iIndex].sensorType;
        		} else {
        			oObject.target={type:'zone',zone:parseInt('' + oResultObject[iIndex].zone,10),group:parseInt('' + oResultObject[iIndex].group,10)};
        			oObject.target_zone=oResultObject[iIndex].zone + '_' + oResultObject[iIndex].group + '_' + oResultObject[iIndex].sensorType
        		}
        		oObject.activevalue=parseFloat((unescape(oResultObject[iIndex].activeValue)).split(';')[1]);
        		oObject.inactivevalue=parseFloat((unescape(oResultObject[iIndex].inactiveValue)).split(';')[1]);
        		oArray.push(oObject);
        	}
            grid.loadParameter(oArray);
        }
        grid.setStatus(_('Ready'));
    },
    loadSensorBase: function () {
    	if (Ext.data.StoreManager.lookup('zoneSensorBase') == null) {
			Ext.create('Ext.data.Store', {
				storeId : 'zoneSensorBase',
				fields : [ 'zone', 'group', 'type','_id','name'],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				}
			});
		}
    	Ext.data.StoreManager.lookup('zoneSensorBase').removeAll();
    	var sString = dss.ajaxSyncRequest('/json/property/query2', {
            'query' : '/apartment/zones/*(ZoneID)/groups/*(group)/sensor/*(*)'
        });
    	var oZoneSensors=[];
        var result = Ext.JSON.decode(sString);
        if (result.result!=null) {
        	for (var sZoneKey in result.result) {
        		var zoneNumber=result.result[sZoneKey].ZoneID;
        		var oZoneObject=result.result[sZoneKey];
        		for (var sGroupID in oZoneObject) {
        			if (oZoneObject[sGroupID].group!=null) {
        				var oGroupObject=oZoneObject[sGroupID];
        				var groupNumber=oGroupObject.group;
        				for (var sTypeID in oGroupObject) {
        					if (oGroupObject[sTypeID].type!=null) {
        						var typeNumber=oGroupObject[sTypeID].type;
        						var name=getNameForSensorType(typeNumber);
        						
        						 if (groupNumber != 0)
        							 name += _(' for ') + dss.staticDataModel.getNameForGroup(zoneNumber,groupNumber); 
                                 if (zoneNumber != 0)
                                	 name += _(" in ") + dss.staticDataModel.getNameForZone(zoneNumber);

        						oZoneSensors.push({
        							zone:zoneNumber,
        							group:groupNumber,
        							type:typeNumber,
        							_id:zoneNumber+'_'+groupNumber+'_'+typeNumber,
        							name:name 
        						})
        					}
        				}
        			}
        		}
        	}        	
        	Ext.data.StoreManager.lookup('zoneSensorBase').add(oZoneSensors);        	        	
        }
    	if (Ext.data.StoreManager.lookup('deviceSensorBase') == null) {
			Ext.create('Ext.data.Store', {
				storeId : 'deviceSensorBase',
				fields : [ 'zone', 'group', 'type','_id','name'],
				proxy : {
					type : 'memory',
					reader : {
						type : 'json',
						root : 'items'
					}
				}
			});
		}
    	Ext.data.StoreManager.lookup('deviceSensorBase').removeAll();
        var oDeviceSensors=[];
        sString = dss.ajaxSyncRequest('/json/property/query2', {
            'query' : '/apartment/zones/zone0/devices/*(dSUID)/sensorInputs/*(type)'
        });
        result = Ext.JSON.decode(sString);
        if (result.result!=null) {
        	for (var sDSUID in result.result) {
        		var sDSUID = result.result[sDSUID].dSUID;
        		for (var sSensor in result.result[sDSUID] ) {
        			if (result.result[sDSUID][sSensor].type!=null) {
        				if (result.result[sDSUID][sSensor].type!=253)
        					if (result.result[sDSUID][sSensor].type!=255) {
        						var oDevice=dss.staticDataModel.getDevice(sDSUID);
        						oDeviceSensors.push({
        							dsid:sDSUID,
        							type:result.result[sDSUID][sSensor].type,
        							_id: sDSUID+'_' + result.result[sDSUID][sSensor].type,
        							name: getNameForSensorType(result.result[sDSUID][sSensor].type) +_(' from ') + oDevice.name
        						});
        					}
        			}
        		}
        	}
        	Ext.data.StoreManager.lookup('deviceSensorBase').add(oDeviceSensors);        	        	

        }
    }
});

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/user-defined-actions.po']);
    var HLEEdit = Ext.create('DSS.addon.HLEEdit');
    HLEEdit.initPage();
});
